import {
  Channel
} from "./chunk-URTIMVUU.js";
import {
  Emitter
} from "./chunk-NLNCD3ES.js";
import {
  uname,
  uuid
} from "./chunk-BMLTMNZA.js";
import {
  useDispose
} from "./chunk-VI7IQ5ZG.js";
import {
  createPromise,
  promisify
} from "./chunk-L324TYGN.js";
import {
  arrayRemoveElement
} from "./chunk-ITJJZ6AV.js";

// src/common/exec/progress.ts
var Progress = class _Progress extends Emitter {
  constructor(opt = {}) {
    super();
    this._isCancelled = false;
    this._resetWhenFinished = true;
    this._children = [];
    this.dispose = useDispose();
    this._totalUnits = opt.totalUnits ?? 0;
    this._completedUnits = opt.completeUnits ?? 0;
    this._resetWhenFinished = opt.resetWhenFinished ?? true;
    this.name = opt.name ?? uname("progress");
    this.dispose.add(async () => {
      for (const child of this._children)
        await child.dispose();
      await this.emit("progressDispose", this);
    });
  }
  /** Notify others about changes and reset on completion, if flag is set. */
  update() {
    void this.emit("progressChanged", this);
    if (this._isCancelled && this._resetWhenFinished) {
      if (this.getTotalUnits() <= this.getCompletedUnits())
        this.reset();
    }
  }
  /** Fresh start */
  reset() {
    if (this._isCancelled) {
      this._isCancelled = false;
      for (const child of this._children)
        child.reset();
      this.update();
    }
  }
  /** Notify and mark as cancelled. May take some time before having an effect. */
  async cancel() {
    if (!this._isCancelled) {
      this._isCancelled = true;
      await this.emit("progressCancelled", this);
      for (const child of this._children)
        await child.cancel();
      this.update();
    }
  }
  /** Add child progress, which count into its parents units. On `dispose` it will auto remove itself from parent. */
  addChild(child) {
    child.on("progressDispose", () => this.removeChild(child));
    child.on("progressChanged", () => this.update());
    if (!this._children.includes(child))
      this._children.push(child);
    this.update();
  }
  /** Create child progress.  */
  createChildProgress(opt) {
    const progress = new _Progress(opt);
    this.addChild(progress);
    return progress;
  }
  removeChild(child) {
    arrayRemoveElement(this._children, child);
    this.update();
  }
  /** Total units including children */
  getTotalUnits() {
    if (this.isIndeterminate())
      return 0;
    let units = this._totalUnits;
    for (const child of this._children)
      units += child.getTotalUnits();
    return units;
  }
  /** Completed units including children */
  getCompletedUnits() {
    if (this.isIndeterminate())
      return 0;
    let units = this._completedUnits;
    for (const child of this._children)
      units += child.getCompletedUnits();
    return units;
  }
  /** Cannot calculate progress, because totalUnits are missing. Special representation in UI. */
  isIndeterminate() {
    return this._totalUnits <= 0 && this._children.length <= 0;
  }
  isCancelled() {
    return this._isCancelled;
  }
  /** Either disposed or all units completed. */
  isFinished() {
    return this.dispose.isDisposed() || !this.isIndeterminate() && this.getTotalUnits() <= this.getCompletedUnits();
  }
  /** Value from 0 to 1, where 1 is 100% completeness. */
  getFraction() {
    if (this.isIndeterminate())
      return 0;
    let value = this.getCompletedUnits() / this.getTotalUnits();
    if (Number.isNaN(value))
      value = 0;
    return Math.min(1, Math.max(0, value));
  }
  getChildrenCount() {
    return this._children.length;
  }
  /** Change total units. */
  setTotalUnits(units, completedUnits) {
    this._totalUnits = units;
    if (completedUnits != null)
      this._completedUnits = completedUnits;
    this.update();
  }
  /** Relatively change total units. */
  incTotalUnits(step = 1) {
    this._totalUnits += step;
    this.update();
  }
  /** Set fixed number of completed units. */
  setCompletetedUnits(units) {
    this._completedUnits = units;
    this.update();
  }
  /** Set to 100% without disposing. */
  setCompleted() {
    this._completedUnits = this._totalUnits;
    this.update();
  }
  /** Dynamically change completed units. */
  incCompletedUnits(step = 1) {
    this._completedUnits += step;
    this.update();
  }
  /** Progress tree to string for debuggin purposes. Consider using `name` attribute of Progress. */
  toString(indent = 0) {
    let s = `${"  ".repeat(indent)}${this.name}: ${this._completedUnits} of ${this._totalUnits} units, ${Math.floor(this.getFraction() * 100)} %, cancel=${this._isCancelled}
`;
    for (const child of this._children)
      s += child.toString(indent + 1);
    if (indent === 0)
      return s.trim();
    return s;
  }
};

// src/common/exec/pool.ts
function usePool(config = {}) {
  const {
    maxParallel = 3,
    name = uname("pool"),
    idConflictResolution = "memoize"
  } = config;
  const events = new Emitter();
  const dispose = useDispose();
  dispose.add(cancelAll);
  const progress = new Progress({ name });
  progress.on("progressCancelled", cancelAll);
  let countMax = 0;
  let countResolved = 0;
  let currentParallel = 0;
  let priority = 0;
  const tasks = {};
  async function waitFinishAll() {
    if (countMax > 0) {
      const [promise, resolve] = createPromise();
      events.once("didFinish", resolve);
      return promise;
    }
  }
  async function cleanupFinished() {
    for (const id of Object.keys(tasks)) {
      const task = tasks[id];
      if (task.state === "finished") {
        await tasks[id].progress.dispose();
        delete tasks[id];
      }
    }
  }
  function didFinish() {
    void events.emit("didFinish");
    countMax = 0;
    countResolved = 0;
    void cleanupFinished();
    progress.reset();
  }
  function didUpdate() {
    let presentMax = 0;
    let presentResolved = 0;
    for (const { max, resolved, state } of Object.values(tasks)) {
      presentMax += max;
      presentResolved += state === "finished" ? max : Math.min(max, resolved);
    }
    void events.emit(
      "didUpdate",
      countMax,
      countResolved,
      presentMax,
      presentResolved
    );
  }
  function performNext() {
    didUpdate();
    if (countMax > 0 && countMax === countResolved)
      didFinish();
    if (currentParallel >= maxParallel)
      return;
    const waitingTasks = Object.values(tasks).filter(
      (t) => t.state === "waiting"
    );
    if (waitingTasks.length > 0) {
      let taskInfo;
      for (const t of waitingTasks) {
        if (t.group != null && Object.values(tasks).some(
          (tt) => tt.state === "running" && tt.id !== t.id && tt.group === t.group
        )) {
          continue;
        }
        if (taskInfo == null || t.priority < taskInfo.priority)
          taskInfo = t;
      }
      if (taskInfo != null) {
        const id = taskInfo.id;
        const done = taskInfo.done;
        taskInfo.state = "running";
        ++currentParallel;
        void events.emit("didStart", id);
        const taskFinished = (result) => {
          if (taskInfo) {
            taskInfo.result = result;
            taskInfo.state = "finished";
            taskInfo.resolved = taskInfo.max;
            taskInfo.progress?.setCompleted();
          }
          --currentParallel;
          ++countResolved;
          performNext();
        };
        taskInfo.task(taskInfo).then((r) => {
          done(r);
          void events.emit("didResolve", id, r);
          taskFinished(r);
        }).catch((err) => {
          done();
          void events.emit("didReject", id, err);
          taskFinished();
        });
      }
    }
  }
  function cancel(id) {
    const taskInfo = tasks[id];
    if (taskInfo && taskInfo.state === "waiting") {
      tasks[id].state = "finished";
      ++countResolved;
      void events.emit("didCancel", id);
      void tasks[id].progress.dispose();
      didUpdate();
    }
  }
  function cancelAll() {
    Object.keys(tasks).forEach(cancel);
  }
  function hasById(id) {
    return tasks[id] != null;
  }
  function enqueue(task, config2 = {}) {
    let done;
    const promise = new Promise((resolve) => done = resolve);
    const id = config2.id ?? uuid();
    if (tasks[id] != null) {
      const resolution = config2.idConflictResolution ?? idConflictResolution;
      if (resolution === "replace") {
        cancel(id);
      } else if (resolution === "memoize" || resolution === "prioritize") {
        const runningTask = tasks[id];
        if (resolution === "prioritize")
          runningTask.priority = ++priority;
        return {
          id,
          promise: (async () => {
            if (runningTask.state === "finished")
              return tasks[id].result;
          })(),
          dispose: () => cancel(id),
          cancel: () => cancel(id)
        };
      } else {
        throw new Error(`Pool task with id=${id} already exists!`);
      }
    }
    const taskProgress = new Progress({
      name: id,
      totalUnits: config2.max ?? 1,
      completeUnits: config2.resolved ?? 0
    });
    progress.addChild(taskProgress);
    tasks[id] = {
      id,
      task,
      priority: ++priority,
      group: config2.group,
      state: "waiting",
      max: config2.max ?? 1,
      resolved: config2.resolved ?? 0,
      done,
      payload: config2.payload,
      progress: taskProgress,
      /** @deprecated should use `.progress` */
      setMax(units) {
        taskProgress.setTotalUnits(units);
        tasks[id].max = units;
        didUpdate();
      },
      /** @deprecated should use `.progress` */
      setResolved(units) {
        taskProgress.setCompletetedUnits(units);
        tasks[id].resolved = units;
        didUpdate();
      },
      /** @deprecated should use `.progress` */
      incResolved(inc = 1) {
        taskProgress.incCompletedUnits(inc);
        tasks[id].resolved += inc;
        didUpdate();
      }
    };
    ++countMax;
    performNext();
    return {
      id,
      promise,
      dispose: () => cancel(id),
      cancel: () => cancel(id)
    };
  }
  return {
    events,
    cancel,
    cancelAll,
    hasById,
    progress,
    enqueue,
    dispose,
    waitFinishAll
  };
}

// src/common/dispose-utils.ts
var noopDisposer = () => {
  const dispose = () => {
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
};
function useTimeout(fn, timeout = 0) {
  let timeoutHandle = setTimeout(fn, timeout);
  const dispose = () => {
    if (timeoutHandle) {
      clearTimeout(timeoutHandle);
      timeoutHandle = void 0;
    }
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
}
function useInterval(fn, interval) {
  let intervalHandle = setInterval(fn, interval);
  const dispose = () => {
    if (intervalHandle) {
      clearInterval(intervalHandle);
      intervalHandle = void 0;
    }
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
}
function useIntervalPause(fn, interval, immediately = false) {
  let intervalHandle;
  let stop = false;
  async function loop(exec = false) {
    if (exec)
      await promisify(fn());
    if (!stop)
      intervalHandle = setTimeout(() => loop(true), interval);
  }
  void loop(immediately);
  const dispose = () => {
    if (intervalHandle) {
      stop = true;
      clearInterval(intervalHandle);
      intervalHandle = void 0;
    }
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
}
function useEventListener(emitter, eventName, fn, ...args) {
  if (emitter == null)
    return noopDisposer();
  if (emitter.on)
    emitter.on(eventName, fn, ...args);
  else if (emitter.addEventListener)
    emitter.addEventListener(eventName, fn, ...args);
  const dispose = () => {
    if (emitter.off)
      emitter.off(eventName, fn, ...args);
    else if (emitter.removeEventListener)
      emitter.removeEventListener(eventName, fn, ...args);
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
}
function useEventListenerOnce(emitter, eventName, fn, ...args) {
  if (emitter == null)
    return noopDisposer();
  if (emitter.on)
    emitter.once(eventName, fn, ...args);
  else if (emitter.addEventListener)
    emitter.addEventListener(eventName, fn, ...args);
  const dispose = () => {
    if (emitter.off)
      emitter.off(eventName, fn, ...args);
    else if (emitter.removeEventListener)
      emitter.removeEventListener(eventName, fn, ...args);
  };
  dispose[Symbol.dispose] = dispose;
  return dispose;
}
function useDisposeWithUtils(config) {
  const dispose = useDispose(config);
  return Object.assign(dispose, {
    timeout: (fn, timeout = 0) => dispose.add(useTimeout(fn, timeout)),
    interval: (fn, interval = 0) => dispose.add(useInterval(fn, interval)),
    intervalPause: (fn, interval = 0) => dispose.add(useIntervalPause(fn, interval)),
    on: (emitter, eventName, fn, ...args) => dispose.add(useEventListener(emitter, eventName, fn, ...args)),
    once: (emitter, eventName, fn, ...args) => dispose.add(useEventListenerOnce(emitter, eventName, fn, ...args))
  });
}

// src/common/msg/channel-resilient.ts
var ResillientChannel = class extends Channel {
  constructor() {
    super(...arguments);
    this.buffer = [];
    this.dispose = useDisposeWithUtils();
  }
  postMessageRaw(data) {
    try {
      if (this.channel?.isConnected) {
        this.channel.postMessage(data);
        return true;
      }
    } catch (err) {
    }
    return false;
  }
  /** Post all buffered messages */
  flushBuffer() {
    while (this.buffer.length) {
      const data = this.buffer.shift();
      if (data && !this.postMessageRaw(data)) {
        this.buffer.unshift(data);
        break;
      }
    }
  }
  /** Reset buffer without force sending */
  emptyBuffer() {
    this.buffer = [];
  }
  /**
   * Set a new channel and flush buffer
   */
  setChannel(channel) {
    void this.dispose();
    this.channel = channel;
    if (channel) {
      this.dispose.add(channel.on("message", (msg) => this.emit("message", msg)));
      if (channel.isConnected)
        this.flushBuffer();
      else
        this.dispose.add(channel.on("connect", () => this.flushBuffer()));
    }
  }
  /** @deprecated use `setChannel(undefined)` */
  deleteChannel() {
    this.setChannel();
  }
  postMessage(data) {
    this.flushBuffer();
    if (!this.postMessageRaw(data))
      this.buffer.push(data);
  }
  get isConnected() {
    return this.channel?.isConnected === true;
  }
};

export {
  ResillientChannel,
  Progress,
  usePool,
  noopDisposer,
  useTimeout,
  useInterval,
  useIntervalPause,
  useEventListener,
  useEventListenerOnce,
  useDisposeWithUtils
};
//# sourceMappingURL=chunk-KDP637GO.js.map