import {
  isUint8Array
} from "./chunk-V5ZQ2SFQ.js";
import {
  toUint8Array
} from "./chunk-EJGW3W3K.js";
import {
  jsonStringifySorted
} from "./chunk-EAYWSLQT.js";

// src/node/fs.ts
import { mkdir, readFile, rm, stat, writeFile } from "node:fs/promises";
import { dirname, join as joinPath, normalize } from "node:path";
import process from "node:process";
function toHumanReadableFilePath(path) {
  const p = normalize(path);
  const h = process.env.HOME;
  if (h && p.startsWith(h))
    return `~${p.slice(h.length)}`;
  return p;
}
async function exists(path) {
  try {
    await stat(path);
  } catch (err) {
    return false;
  }
  return true;
}
function isHiddenPath(path) {
  return path.startsWith(".") || path.includes("/.");
}
async function ensureFolder(...parts) {
  const path = joinPath(...parts);
  if (!await exists(path))
    await mkdir(path, { recursive: true });
  return path;
}
async function ensureFolderForFile(...parts) {
  const path = dirname(joinPath(...parts));
  if (!await exists(path))
    await mkdir(path, { recursive: true });
  return path;
}
async function removeFolder(...parts) {
  const path = joinPath(...parts);
  if (await exists(path))
    await rm(path, { recursive: true });
  return path;
}
async function readText(...parts) {
  const path = joinPath(...parts);
  if (await exists(path))
    return await readFile(path, "utf-8");
}
async function readJson(...parts) {
  const content = await readText(...parts);
  if (content != null) {
    try {
      return JSON.parse(content);
    } catch (err) {
    }
  }
}
async function readBin(...parts) {
  const path = joinPath(...parts);
  if (await exists(path))
    return toUint8Array(await readFile(path));
}
async function readData(...parts) {
  const path = joinPath(...parts);
  if (await exists(path))
    return await readFile(path);
}
async function writeText(path, content, createFolders = false) {
  if (createFolders)
    await ensureFolderForFile(path);
  await writeFile(path, content, "utf-8");
}
async function writeData(path, content, createFolders = false) {
  if (createFolders)
    await ensureFolderForFile(path);
  const data = isUint8Array(content) ? content : JSON.stringify(content);
  await writeFile(path, data);
}
async function writeBin(path, content, info = {}) {
  const { createFolders = false } = info;
  if (createFolders)
    await ensureFolderForFile(path);
  await writeFile(path, content);
}
async function writeJson(path, content, info = {}) {
  const { createFolders = false, pretty = false } = info;
  await writeText(path, jsonStringifySorted(content, pretty ? 2 : void 0), createFolders);
}

export {
  toHumanReadableFilePath,
  exists,
  isHiddenPath,
  ensureFolder,
  ensureFolderForFile,
  removeFolder,
  readText,
  readJson,
  readBin,
  readData,
  writeText,
  writeData,
  writeBin,
  writeJson
};
//# sourceMappingURL=chunk-QU4LUB3H.js.map